/* This file is part of the KDE project
   Copyright (C) 2003 Lucijan Busch <lucijan@gmx.at>
   Copyright (C) 2004-2005 Jaroslaw Staniek <js@iidea.pl>
   Copyright (C) 2005 Cedric Pasteur <cedric.pasteur@free.fr>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef KEXIEDITOR_H
#define KEXIEDITOR_H

#include <qwidget.h>
#include "kexiviewbase.h"

class KTextEdit;
class KexiEditorPrivate;

namespace KTextEditor
{
	class Document;
	class View;
}

//! An text editor view that uses both KTextEditor and KTextEdit
/*! It is used for SQL and script editor. */
class KEXIEXTWIDGETS_EXPORT KexiEditor : public KexiViewBase
{
	Q_OBJECT

	public:
		KexiEditor(KexiMainWindow *mainWin, QWidget *parent, const char *name = 0);
		virtual ~KexiEditor();

		QString text();
		void jump(int character);

		//void installEventFilter ( const QObject * filterObj );
		//virtual bool eventFilter(QObject *o, QEvent *ev);

	public slots:
		/*! Sets editor's text to \a text. 'Dirty' flag remains unchanged. */
		void setText(const QString &text);

	protected:
		virtual void updateActions(bool activated);

		// functions used by subclasses to acces private pointers
#ifdef KTEXTEDIT_BASED_SQL_EDITOR
#else
		KTextEditor::Document*  document();
		KTextEditor::View*   docView();
#endif

	signals:
		void textChanged();

	private:
		KexiEditorPrivate *d;
};

#endif
